@@ -1,5 +1,6 @@
 import sys
 from typing import Union, Tuple, Any, Dict, Optional, Sequence, cast
+from enum import Enum
 import collections
 from abc import ABC
 
@@ -25,9 +26,11 @@ AbstractType = Union[IsDataclass, IsTypingType, type]
 
 def _classifier(T):
     """Decorator populating :_classifiers:"""
+
     def decorator(f):
         _classifiers.append((T, f))
         return f
+
     return decorator
 
 
@@ -51,8 +54,7 @@ def classify_tuple(T: AbstractType) -> bool:
 @_classifier(Sequence)
 def classify_seq(T: AbstractType) -> bool:
     seq_type = getattr(T, __origin_attr__, None)
-    return (isinstance(seq_type, type) and
-            issubclass(seq_type, collections.abc.Sequence))
+    return isinstance(seq_type, type) and issubclass(seq_type, collections.abc.Sequence)
 
 
 @_classifier(IsDataclass)
@@ -62,8 +64,14 @@ def classify_dataclass(T: type) -> bool:
 
 @_classifier(Optional)
 def classify_optional(T: AbstractType) -> bool:
-    return (getattr(T, "__origin__", None) is Union and
-            is_optional(cast(IsTypingType, T)))
+    return getattr(T, "__origin__", None) is Union and is_optional(
+        cast(IsTypingType, T)
+    )
+
+
+@_classifier(Enum)
+def classify_enum(T: AbstractType) -> bool:
+    return isinstance(T, type) and issubclass(T, Enum)
 
 
 def is_optional(T: IsTypingType) -> bool:
